/* GCC */
//#include<iostream>
#include<fstream>

// Burnside lemma FTW!

using namespace std;

#define MAX_FACES 24
#define BASE 10
#define MAX_DIGITS 20

int faces, colors;
bool matrix[MAX_FACES][MAX_FACES];

struct LongInt {
    int d[MAX_DIGITS];
    
    LongInt() {
        for (int i = 0; i < MAX_DIGITS; i++) d[i] = 0;
    }
    LongInt(const LongInt& a) {
        for (int i = 0; i < MAX_DIGITS; i++) d[i] = a.d[i];
    }
};

void longPrint(LongInt a, ofstream& fout) {
    bool printing = false;
    for (int i = MAX_DIGITS - 1; i >=0; i--) {
        printing |= (a.d[i] != 0);
        if (printing) fout << a.d[i];
    }
    if (!printing) fout << "0";
    fout << "\n";
}

LongInt longAdd(LongInt a, LongInt b) {
    LongInt res;
    int overflow = 0;
    for (int i = 0; i < MAX_DIGITS; i++) {
        res.d[i] = (a.d[i] + b.d[i] + overflow) % BASE;
        overflow = (a.d[i] + b.d[i] + overflow) / BASE;
    }
    return res;
}

LongInt longDivide(LongInt a, int b) {
    LongInt res;

    int t = 0;
    for (int i = MAX_DIGITS - 1; i >= 0; i--) {
        t *= BASE;
        t += a.d[i];
        res.d[i] = t / b;
        t %= b;
    }
    return res;
}

LongInt longMultiply(LongInt a, int b) {
    LongInt res;
    int overflow = 0;
    for (int i = 0; i < MAX_DIGITS; i++) {
        res.d[i] = (a.d[i] * b + overflow) % BASE;
        overflow = (a.d[i] * b + overflow) / BASE;
    }
    return res;
}

LongInt longPower(int a, int b) {
    LongInt res;
    res.d[0] = 1;
    for (int i = 0; i < b; i++) res = longMultiply(res, a);
    return res;
}

void read() {
    for (int i = 0; i < MAX_FACES; i++) {
        for (int j = 0; j < MAX_FACES; j++) {
            matrix[i][j] = 0;
        }
    }
    
    ifstream fin("farben.in");
    char s[MAX_FACES * 4];
    
    fin.getline(s, 4);
    colors = 0;
    for (int i = 0; i < strlen(s); i++) {
        colors *= 10;
        colors += s[i] - '0';
    }
    
    fin.getline(s, MAX_FACES * 4);
    faces = 0;
    while (s[0] != 0) {
        int t = 0;
        for (int i = 0; i <= strlen(s); i++) {
            if (s[i] == ' ' || s[i] == 0) {
                matrix[faces][t - 1] = 1;
                t = 0;
            } else {
                t *= 10;
                t += s[i] - '0';
            }
        }
        faces++;
        fin.getline(s, 75);
    }
    fin.close();
}

int findLoops(int p[MAX_FACES], int n) {
    bool visited[MAX_FACES];
    for (int i = 0; i < n; i++) visited[i] = 0;
    
    int loops = 0;
    for (int i = 0; i < n; i++) {
        if (!visited[i]) {
            for (int j = i; !visited[j]; j = p[j]) visited[j] = 1;
            loops++;
        }
    }
    
    return loops;
}

/*void printp(int p[MAX_FACES], int n) {
    for (int i = 0; i < n; i++) {
        cout << i << "\t -> \t" << p[i] << "\n";
    }
    cout << "\n";
}*/

void findSymmetries(int p[MAX_FACES], int n, int& symmetries, LongInt& sum) {
    if (n == faces) {
        //cout << "Symmetry (" << findLoops(p, n) << " loops):\n";
        //printp(p, n);
        symmetries++;
        sum = longAdd(sum, longPower(colors, findLoops(p, n)));
    } else {
        for (p[n] = 0; p[n] < faces; p[n]++) {
            bool ok = true;
            for (int i = 0; i < n && ok; i++) {
                if (p[i] == p[n] || matrix[n][i] != matrix[p[n]][p[i]]) ok = false;
            }
            if (ok) {
                findSymmetries(p, n + 1, symmetries, sum);
            }
        }
    }
}

LongInt burnside() {
    int p[MAX_FACES];
    int symmetries = 0;
    LongInt sum;
    findSymmetries(p, 0, symmetries, sum);
    //cout << sum << "/" << symmetries << " = ";
    return longDivide(sum, symmetries);
}

int reorderFaces() {
    int a = 0;
    int b;
    for (int i = 1; i < faces; i++) {
        if (matrix[i][0]) {
            b = i;
            break;
        }
    }
    for (int i = 0; i < faces; i++) {
        bool t = matrix[a][i];
        matrix[a][i] = matrix[b][i];
        matrix[b][i] = t;
        
        t = matrix[i][a];
        matrix[i][a] = matrix[i][b];
        matrix[i][b] = t;
    }
}

int main() {
    read();
    reorderFaces();

    ofstream fout("farben.out");
    longPrint(burnside(), fout);
    fout.close();

    //cin.get();
    //cin.get();

    return 0;
}
